<!DOCTYPE html>
<html>
  <head>
    <title>Pricer for WoW Auctions portal</title>
    <meta charset="utf-8">    
    <script src="https://code.jquery.com/jquery-3.3.1.slim.min.js" integrity="sha384-q8i/X+965DzO0rT7abK41JStQIAqVgRVzpbzo5smXKp4YfRvH+8abtTE1Pi6jizo" crossorigin="anonymous"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.6/umd/popper.min.js" integrity="sha384-wHAiFfRlMFy6i5SRaxvfOCifBUQy1xHdJ/yoi7FRNXMRBu5WHdZYu1hA6ZOblgut" crossorigin="anonymous"></script>
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.2.1/css/bootstrap.min.css" integrity="sha384-GJzZqFGwb1QTTN6wy59ffF1BuGJpLSa9DkKMp0DgiMDm4iYMj70gZWKYbI706tWS" crossorigin="anonymous">
    <script src="https://stackpath.bootstrapcdn.com/bootstrap/4.2.1/js/bootstrap.min.js" integrity="sha384-B0UglyR+jN6CkvvICOB2joaf5I4l3gm9GU6Hc1og6Ls7i6U/mkkaduKaBhlAXv9k" crossorigin="anonymous"></script>
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='style.css') }}">
    <link href="//maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css">

    <link rel="stylesheet"
          href="//cdnjs.cloudflare.com/ajax/libs/highlight.js/9.15.9/styles/default.min.css">
    <script src="//cdnjs.cloudflare.com/ajax/libs/highlight.js/9.15.9/highlight.min.js"></script>
    <script>hljs.initHighlightingOnLoad();</script>

  </head>
<body class="bg-light text-dark">

<div class="container-fluid">
  <div class="row">
    <div class="col-lg-4">
      <h2><a href="{{url_for('home')}}">Pricer for WoW auctions</a></h2>
    </div>
    <div class="col-lg-8">
      Item details | Total profits
      <a href="{{ url_for('trigger_booty_bay')}}" class="btn btn-primary">Trigger Booty Bay</a>
      <a href="{{ url_for('run_analytics')}}" class="btn btn-primary">Run Analytics</a>
    </div>
  </div>
</div>

<div class="container-fluid">
  <div class="row">
    <div class="col-lg-3">
      <div class="shadow-sm p-1 bg-white rounded">
        <h3>Item details</h3>
        <ul>
          {% for item_name in user_items() %}
          <li>
            <img src="{{ url_for('item_icons', filename=item_name) }}" width="24">
            <a href="{{ url_for('item_report', item_name=item_name) }}">{{item_name}}</a>
          </li>
          {% endfor %}
        </ul>
      </div>
    </div>
    <div class="col-lg-9">
      <div class="shadow-sm p-1 bg-white rounded">
      {% block content %}
      {% endblock %}
      </div>
    </div>
  </div>
</div>

<script src="{{ url_for('static', filename='scripts.js') }}"></script>

</body>
</html>